/*
/ Dateiname   : display.c
/ Versionsnr. : 1.0.0
/ Datum       : 15.01.2011 
/  
/ Autor		  : Pollin-Team
*/

 
#include <avr/io.h>
#include <util/delay.h>
#include "display.h"

 
 
/* -------------------------------------------------------------------------------------
   Initialisierungsroutine fr das Display 
   Parametereinstellungen: 4 Bit Modus aktivieren                   0x02
  	      			       2 Zeilen, Zeichen 5x7                    0x28	
                           Display ein, Cursor aus, kein Blinken    0x0C 
                           Speicher inkrementieren, kein Scrollen   0x06
--------------------------------------------------------------------------------------*/ 
void InitDisplay(void)
{
    // Setze Port-Direction-Register fr DATA4 bis DATA7 sowie RS und E auf Ausgang
    DDR_DISPLAY_DATA |= ((1<<BIT_DISPLAY_DATA4) | (1<<BIT_DISPLAY_DATA5) | (1<<BIT_DISPLAY_DATA6) | (1<<BIT_DISPLAY_DATA7)); 
    DDR_DISPLAY_RS   |= (1<<BIT_DISPLAY_RS);      
    DDR_DISPLAY_E    |= (1<<BIT_DISPLAY_E);       

 
    _delay_us(15000);		   // --- Warte 15 ms nach Power-On ---

   
   // Setze RS auf low um Daten ins Steuerregister zu schreiben
   PORT_DISPLAY_RS &= ~(1<<BIT_DISPLAY_RS);     				    // Setze RS auf 0; Steuerregister
 

   // Initialisierung erfolgt noch mit 8 Bit
   // Function set 0x03;
   PORT_DISPLAY_DATA |=  (1<<BIT_DISPLAY_DATA4);	    			// Setze DB4 auf 1
   PORT_DISPLAY_DATA |=  (1<<BIT_DISPLAY_DATA5);	    			// Setze DB5 auf 1
   PORT_DISPLAY_DATA &= ~(1<<BIT_DISPLAY_DATA6);					// Setze DB6 auf 0
   PORT_DISPLAY_DATA &= ~(1<<BIT_DISPLAY_DATA7);					// Setze DB7 auf 0

   PORT_DISPLAY_RS   &= ~(1<<BIT_DISPLAY_RS);				        // Setze RS  auf 0   


   // --- Enable-Puls fr Datenbernahme erzeugen ---
   PORT_DISPLAY_E |=  (1<<BIT_DISPLAY_E);
   _delay_us(10);                

   PORT_DISPLAY_E &= ~(1<<BIT_DISPLAY_E);
 
   _delay_us(4100);		      

   
   // --- Der vorhergehende Befehl wir wiederholt gesendet --- 
   // --- Enable-Puls fr Datenbernahme erzeugen ---
   PORT_DISPLAY_E |=  (1<<BIT_DISPLAY_E);
   _delay_us(10);                
   PORT_DISPLAY_E &= ~(1<<BIT_DISPLAY_E);
 
   _delay_us(100);		    


   // --- Der vorhergehende Befehl wir zum dritten Mal gesendet --- 
   // --- Enable-Puls fr Datenbernahme erzeugen ---
   PORT_DISPLAY_E |=  (1<<BIT_DISPLAY_E);
   _delay_us(2);                
   PORT_DISPLAY_E &= ~(1<<BIT_DISPLAY_E);
   _delay_us(100);  		 
   
 
   // Sende Kommando 4 Bit Modus zu aktivieren
   // Function set 0x02;
   PORT_DISPLAY_DATA &= ~(1<<BIT_DISPLAY_DATA4);	    			// Setze DB4 auf 0
   PORT_DISPLAY_DATA |=  (1<<BIT_DISPLAY_DATA5);	    			// Setze DB5 auf 1 
   PORT_DISPLAY_DATA &= ~(1<<BIT_DISPLAY_DATA6);	    			// Setze DB6 auf 0
   PORT_DISPLAY_DATA &= ~(1<<BIT_DISPLAY_DATA7);	    			// Setze DB7 auf 0   

   // --- Enable-Puls fr Datenbernahme erzeugen ---
   PORT_DISPLAY_E |=  (1<<BIT_DISPLAY_E);
   _delay_us(10);                

   PORT_DISPLAY_E &= ~(1<<BIT_DISPLAY_E);
   _delay_us(1000);
 
   WriteDisplayControl(0x28);								    	// 4Bit / 2 Zeilen / 5x7
   WriteDisplayControl(0x0C); 								    	// Display ein / Cursor aus / kein Blinken	
   WriteDisplayControl(0x06);										// inkrement / kein Scrollen
 
   ClearDisplay();
}



/* -------------------------------------------------------------------------------------
   Funktion :	      Sendet einen Befehl bzw Daten im 4-Bit-Modus an das Steuer oder
					  Datenregister des Displays entsprechend der Funktion WriteDisplayControl() 
					  oder WriteDisplayData() die diese Funktion aufruft.  
   Parameterbergabe: Input     : data     Byte das an das Display gesendet werden soll
                      Return    : -
--------------------------------------------------------------------------------------*/ 
void WriteDisplay(unsigned char data)
{

   // ---------- sendet die ersten 4 Bits an das Display (MSB) ------------
   // ----------      Daten am Pin 4/5/6/7 lschen             ------------
   PORT_DISPLAY_DATA &= ~((1<<BIT_DISPLAY_DATA4) | (1<<BIT_DISPLAY_DATA5) | (1<<BIT_DISPLAY_DATA6) | (1<<BIT_DISPLAY_DATA7));					

   if(data & 0x10) PORT_DISPLAY_DATA |= (1<<BIT_DISPLAY_DATA4);	// Daten am Pin 4 setzen
   if(data & 0x20) PORT_DISPLAY_DATA |= (1<<BIT_DISPLAY_DATA5);	// Daten am Pin 5 setzen
   if(data & 0x40) PORT_DISPLAY_DATA |= (1<<BIT_DISPLAY_DATA6);	// Daten am Pin 6 setzen
   if(data & 0x80) PORT_DISPLAY_DATA |= (1<<BIT_DISPLAY_DATA7);	// Daten am Pin 7 setzen

   // --- Enable-Puls fr Datenbername erzeugen ---
   PORT_DISPLAY_E |=  (1<<BIT_DISPLAY_E);
   _delay_us(500);                
   PORT_DISPLAY_E &= ~(1<<BIT_DISPLAY_E);

   // ---------- sendet die zweiten 4 Bits an das Display (LSB) ------------
   // ----------       Daten am Pin 4/5/6/7 lschen             ------------
   PORT_DISPLAY_DATA &= ~((1<<BIT_DISPLAY_DATA4) | (1<<BIT_DISPLAY_DATA5) | (1<<BIT_DISPLAY_DATA6) | (1<<BIT_DISPLAY_DATA7));					

   if(data & 0x01) PORT_DISPLAY_DATA |= (1<<BIT_DISPLAY_DATA4);	// Daten am Pin 4 setzen
   if(data & 0x02) PORT_DISPLAY_DATA |= (1<<BIT_DISPLAY_DATA5);	// Daten am Pin 5 setzen
   if(data & 0x04) PORT_DISPLAY_DATA |= (1<<BIT_DISPLAY_DATA6);	// Daten am Pin 6 setzen
   if(data & 0x08) PORT_DISPLAY_DATA |= (1<<BIT_DISPLAY_DATA7);	// Daten am Pin 7 setzen

   // --- Enable-Puls fr Datenbername erzeugen ---
   PORT_DISPLAY_E |=  (1<<BIT_DISPLAY_E);
   _delay_us(500);                
   PORT_DISPLAY_E &= ~(1<<BIT_DISPLAY_E);
}


 
/* -------------------------------------------------------------------------------------
   Funktion :		  Sendet einen Befehl im 4-Bit-Modus an das Steuerregister des Displays 
   Parameterbergabe: Input     : data     Byte das an das Display gesendet werden soll
					  Return    : -
--------------------------------------------------------------------------------------*/ 
void WriteDisplayControl(unsigned char data)
{
   _delay_us(500);
 
   PORT_DISPLAY_RS &= ~(1<<BIT_DISPLAY_RS);        				// RS auf 0 setzen
   
   WriteDisplay(data);
}



/* -------------------------------------------------------------------------------------
   Funktion :		  Sendet Daten im 4-Bit-Modus an das Datenregister des Displays 
   Parameterbergabe: Input     : data     Byte das an das Display gesendet werden soll
                      Return    : -
--------------------------------------------------------------------------------------*/ 
void WriteDisplayData(unsigned char data)
{
   _delay_us(500);
 
   PORT_DISPLAY_RS |=  (1<<BIT_DISPLAY_RS);      					// RS auf 1 setzen
 
   WriteDisplay(data);
}



/* -------------------------------------------------------------------------------------
   Funktion :         Diese Routine lscht den am Display angezeigten Text 
   Parameterbergabe: Input     : - 
                      Return    : -
--------------------------------------------------------------------------------------*/ 
void ClearDisplay(void)
{
   WriteDisplayControl(CLEAR_DISPLAY);

   _delay_us(5000); 
}


/* -------------------------------------------------------------------------------------
   Funktion :         Diese Routine positioniert den Cursor in der Zeile y (1..2) und
					  Spalte x (0..8) 
   Parameterbergabe: Input     : y           Zeile  (1..2) 
                                  x           Spalte (0..8)
                      Return    : -
--------------------------------------------------------------------------------------*/ 
void SetDisplayCursor(unsigned char  y, unsigned char x)
{
    unsigned char tmp; 

	if(y==1)
		tmp=0x80+0x00+x;     // 1. Zeile
    else 
		tmp=0x80+0x40+x;     // 2. Zeile
  
    WriteDisplayControl(tmp);
}
 

/* -------------------------------------------------------------------------------------
   Funktion :   	  Diese Routine sendet einen String an das Dsipay  
   Parameterbergabe: Input     : *data       der zu sendende Text 
                      Return    : -
--------------------------------------------------------------------------------------*/ 
void WriteDisplayString(char *data)
{
    while(*data)
    {
        WriteDisplayData(*data);
        data++;
    }
}



/* -------------------------------------------------------------------------------------
   Funktion :              Diese Funktion setzt die Pins des C die fr die Ansteuerung 
                           des Displays bentigt werden, whrend des Sleep-Modus des C,
                           in einen hochohmigen Zustand und tragen somit zur Engergieeinsparung 
						   im Sleep-Modus bei.
   Parametereinstellungen: Input     : - 
						   Return    : -
--------------------------------------------------------------------------------------*/ 
void SleepDisplay(void)
{
    // Setze Port-Direction-Register fr DATA4 bis DATA7 sowie RS und E auf Eingang
    DDR_DISPLAY_DATA &= ~((1<<BIT_DISPLAY_DATA4) | (1<<BIT_DISPLAY_DATA5) | (1<<BIT_DISPLAY_DATA6) | (1<<BIT_DISPLAY_DATA7)); 
    DDR_DISPLAY_RS   &= ~(1<<BIT_DISPLAY_RS);      
    DDR_DISPLAY_E    &= ~(1<<BIT_DISPLAY_E);       

    // Setze Port-Pins auf Low-Pegel fr DATA4 bis DATA7 sowie RS und E
    PORT_DISPLAY_DATA &= ~((1<<BIT_DISPLAY_DATA4) | (1<<BIT_DISPLAY_DATA5) | (1<<BIT_DISPLAY_DATA6) | (1<<BIT_DISPLAY_DATA7)); 
    PORT_DISPLAY_RS   &= ~(1<<BIT_DISPLAY_RS);      
    PORT_DISPLAY_E    &= ~(1<<BIT_DISPLAY_E);       
}



